/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include <exception>
#include <stdexcept>
#include <iostream>
#include <cstdlib>
#include <string>
#include <fstream>
#include <vector>
#include <set>
#include <sstream>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <cassert>
#include <signal.h>


using namespace std;


static GtkWidget *progressbar1;
static bool initialized = false;
static double fraction = 0;


void update_progress_bar (double fraction_up)
{
  fraction += fraction_up;
  if (fraction > 1)
    fraction = 1;
  if (initialized)
    gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar1), fraction);
}


void signal_handler (int signum) {
  switch (signum) {
    case SIGHUP  : { gtk_main_quit ();           break; }
    case SIGUSR1 : { update_progress_bar (0.01); break; }
    case SIGUSR2 : { update_progress_bar (0.1);  break; }
  }
}



int main (int argc, char *argv[])
/*
This splashscreen is an independent program, independent from bibledit.
This is because updating a splashscreen while either building a window, or
destroying it, caused error in Gtk. An independent program solves this.
When we receive SIGHUP we terminate the program.
When we receive SIGUSR1, we increase the progressbar with 1 percent.
When we receive SIGUSR2, we increase the progressbar with 10 percent.
*/
{
  // Check we have the right number of arguments passed to the program.
  if (argc != 2) {
    cerr << "Usage: bibledit.splash text" << endl;
    return 1;    
  }
  
  // Set up signal handling.
  struct sigaction sa;
  memset (&sa, 0, sizeof(sa));
  sa.sa_handler = signal_handler;
  bool error = false;
  if (sigaction (SIGHUP, &sa, NULL)) {
    error = true;
  }
  if (sigaction (SIGUSR1, &sa, NULL)) {
    error = true;
  }
  if (sigaction (SIGUSR2, &sa, NULL)) {
    error = true;
  }
  if (error) {
    cerr << "Could not set up signal handling" << endl;
    return 1;
  }

  // Graphical stuff.  
  gtk_set_locale ();
  gtk_init (&argc, &argv);

  GtkWidget *splashscreen;
  GtkWidget *fixed1;
  GtkWidget *infolabel;

  splashscreen = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position (GTK_WINDOW (splashscreen), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (splashscreen), 300, 100);
  gtk_window_set_type_hint (GTK_WINDOW (splashscreen), GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
  gtk_window_set_gravity (GTK_WINDOW (splashscreen), GDK_GRAVITY_CENTER);

  fixed1 = gtk_fixed_new ();
  gtk_widget_show (fixed1);
  gtk_container_add (GTK_CONTAINER (splashscreen), fixed1);

  progressbar1 = gtk_progress_bar_new ();
  gtk_widget_show (progressbar1);
  gtk_fixed_put (GTK_FIXED (fixed1), progressbar1, 8, 56);
  gtk_widget_set_size_request (progressbar1, 280, 10);

  // The text in the info label is taken from the commandline.
  infolabel = gtk_label_new (argv[1]);
  gtk_widget_show (infolabel);
  gtk_fixed_put (GTK_FIXED (fixed1), infolabel, 8, 24);
  gtk_widget_set_size_request (infolabel, 280, 17);
  gtk_label_set_justify (GTK_LABEL (infolabel), GTK_JUSTIFY_CENTER);
  gtk_misc_set_alignment (GTK_MISC (infolabel), 0.5, 0);

  gtk_widget_show (splashscreen);
  
  // Indicate we're ready to update graphical system.
  // At times the graphical systme is slow in starting up. Yet the program is
  // already receiving signals to update it. The result of these signals is now
  // stored until the system is initialized, and then it updates the gui.
  initialized = true;
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar1), fraction);

  gtk_main ();
  return 0;

}
