/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogcompare.h"
#include "projects.h"
#include "projectutils.h"
#include "directories.h"
#include "combobox.h"
#include "keyboard.h"


CompareDialog::CompareDialog (Configuration * configuration)
{
  // Store variables
  myconfiguration = configuration;
  
  comparedialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (comparedialog), "Compare project");
  gtk_window_set_position (GTK_WINDOW (comparedialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (comparedialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (comparedialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Select a project to compare the current project with\n"
                          "and then press OK");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 4);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  combobox1 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox1);
  gtk_box_pack_start (GTK_BOX (vbox1), combobox1, FALSE, FALSE, 4);

  // Get all projects
  // Fill the combo box 
  Projects projects (directories_get_projects ());
  combobox_set_strings (combobox1, projects.projects);
  combobox_set_string (combobox1, configuration->project_to_compare_with);

  checkbutton1 = gtk_check_button_new_with_mnemonic ("_Print changed verses only");
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (vbox1), checkbutton1, FALSE, FALSE, 0);
  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), myconfiguration->print_changes_only);

  dialog_action_area1 = GTK_DIALOG (comparedialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (comparedialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (comparedialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);
  
  g_signal_connect ((gpointer) combobox1, "key_press_event", G_CALLBACK (on_combobox1_key_press_event), gpointer (this));
  g_signal_connect ((gpointer) okbutton, "clicked", G_CALLBACK (static_on_okbutton_clicked), gpointer (this));

  gtk_widget_grab_focus (combobox1);
  gtk_widget_grab_default (okbutton);
}


CompareDialog::~CompareDialog ()
{
  gtk_widget_destroy (comparedialog);
}


int CompareDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (comparedialog));
}


bool CompareDialog::on_combobox1_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  return ((CompareDialog *) user_data)->on_combobox1_key_press (event);
}


bool CompareDialog::on_combobox1_key_press (GdkEventKey *event)
{
  if (keyboard_enter_pressed (event)) {
    on_okbutton_clicked ();
    gtk_dialog_response (GTK_DIALOG (comparedialog), GTK_RESPONSE_OK);
    return true;
  }
  return false;
}


void CompareDialog::static_on_okbutton_clicked (GtkButton * button, gpointer user_data)
{
  ((CompareDialog *) user_data)->on_okbutton_clicked ();
}


void
CompareDialog::on_okbutton_clicked ()
{
  // Store user entered data.
  myconfiguration->project_to_compare_with = combobox_get_active_string (combobox1);
  myconfiguration->print_changes_only = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton1));
}
