/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogimportsfm.h"
#include "usfmtools.h"
#include "bible.h"
#include "scripture.h"
#include "utilities.h"
#include "dialogwait.h"
#include "utilities.h"
#include "book.h"
#include "projectutils.h"
#include "usfmtools.h"
#include "dialogselectbooks.h"
#include "gwrappers.h"


#define TEMP_DIR "bibledit.import"


ImportSfmDialog::ImportSfmDialog (Configuration * configuration, Session * session)
{
  // Variables
  myconfiguration = configuration;
  mysession = session;

  // Make the temporary director where we're going to place the files to import.
  create_directory (temporary_file (TEMP_DIR));
  
  importsfmdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (importsfmdialog), "Import SFM text");
  gtk_window_set_position (GTK_WINDOW (importsfmdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (importsfmdialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (importsfmdialog), 480, 320);
  gtk_window_set_type_hint (GTK_WINDOW (importsfmdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (importsfmdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  notebook = gtk_notebook_new ();
  gtk_widget_show (notebook);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook, TRUE, TRUE, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook), vbox4);

  label13 = gtk_label_new ("You are going to import text with Standard Format Markers into your project.\n\nClick Forward to continue.");
  gtk_widget_show (label13);
  gtk_box_pack_start (GTK_BOX (vbox4), label13, FALSE, FALSE, 5);

  label1 = gtk_label_new ("");
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0), label1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (notebook), vbox2);

  label6 = gtk_label_new ("Select the directory that has the files to import");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (vbox2), label6, FALSE, FALSE, 10);

  select_directory_button = gtk_button_new ();
  gtk_widget_show (select_directory_button);
  gtk_box_pack_start (GTK_BOX (vbox2), select_directory_button, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (select_directory_button), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

  label8 = gtk_label_new_with_mnemonic ("");
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox2), label8, FALSE, FALSE, 0);

  number_of_files_label = gtk_label_new ("Number of files\nNumber of SFM texts");
  gtk_widget_show (number_of_files_label);
  gtk_box_pack_start (GTK_BOX (vbox2), number_of_files_label, FALSE, FALSE, 10);

  label4 = gtk_label_new ("");
  gtk_widget_show (label4);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1), label4);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (notebook), scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (viewport1), vbox3);

  label10 = gtk_label_new ("Here is the list of files that will be used to import text from:");
  gtk_widget_show (label10);
  gtk_box_pack_start (GTK_BOX (vbox3), label10, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (label10), 0.02, 0.5);

  list_of_files_label = gtk_label_new ("");
  gtk_widget_show (list_of_files_label);
  gtk_box_pack_start (GTK_BOX (vbox3), list_of_files_label, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (list_of_files_label), 0.02, 0.5);

  label11 = gtk_label_new ("If you are not satisfied with it, add some files to the directory, or remove some.");
  gtk_widget_show (label11);
  gtk_box_pack_start (GTK_BOX (vbox3), label11, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (label11), 0.02, 0.5);

  label2 = gtk_label_new ("");
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 2), label2);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_container_add (GTK_CONTAINER (notebook), scrolledwindow2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport2 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport2);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), viewport2);

  summary1label = gtk_label_new ("");
  gtk_widget_show (summary1label);
  gtk_container_add (GTK_CONTAINER (viewport2), summary1label);
  gtk_misc_set_alignment (GTK_MISC (summary1label), 0, 0.5);

  label16 = gtk_label_new ("");
  gtk_widget_show (label16);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 3), label16);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (notebook), vbox5);

  label18 = gtk_label_new ("We now know which files to import.\nThe next thing is to check if the files are in Unicode format.\n\nIf a file is found that is not in Unicode format, you will be asked to convert it.");
  gtk_widget_show (label18);
  gtk_box_pack_start (GTK_BOX (vbox5), label18, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);

  label15 = gtk_label_new ("");
  gtk_widget_show (label15);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 4), label15);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox6);
  gtk_container_add (GTK_CONTAINER (notebook), vbox6);

  label19 = gtk_label_new ("Everything is now in Unicode format.\n\nNext you are given an opportunity to choose which books to import.");
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (vbox6), label19, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

  label14 = gtk_label_new ("");
  gtk_widget_show (label14);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 5), label14);

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (notebook), vbox7);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox3, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox3), 4);

  change_selection_button = gtk_button_new ();
  gtk_widget_show (change_selection_button);
  gtk_box_pack_start (GTK_BOX (hbox3), change_selection_button, FALSE, FALSE, 0);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (change_selection_button), alignment2);

  hbox4 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox4);

  image2 = gtk_image_new_from_stock ("gtk-properties", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_box_pack_start (GTK_BOX (hbox4), image2, FALSE, FALSE, 0);

  label25 = gtk_label_new_with_mnemonic ("_Select which books will be imported");
  gtk_widget_show (label25);
  gtk_box_pack_start (GTK_BOX (hbox4), label25, FALSE, FALSE, 0);

  label20 = gtk_label_new ("");
  gtk_widget_show (label20);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 6), label20);

  vbox8 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (notebook), vbox8);

  label23 = gtk_label_new ("All preparation is now ready.\n\nPress Apply to actually import the files.");
  gtk_widget_show (label23);
  gtk_box_pack_start (GTK_BOX (vbox8), label23, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);

  label22 = gtk_label_new ("");
  gtk_widget_show (label22);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 7), label22);

  hbox1 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 4);

  backbutton = gtk_button_new_from_stock ("gtk-go-back");
  gtk_widget_show (backbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), backbutton, FALSE, FALSE, 0);

  forwardbutton = gtk_button_new_from_stock ("gtk-go-forward");
  gtk_widget_show (forwardbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), forwardbutton, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (forwardbutton, GTK_CAN_DEFAULT);

  dialog_action_area1 = GTK_DIALOG (importsfmdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (importsfmdialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  applybutton = gtk_button_new_from_stock ("gtk-apply");
  gtk_widget_show (applybutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (importsfmdialog), applybutton, GTK_RESPONSE_APPLY);
  gtk_widget_set_sensitive (applybutton, FALSE);
  GTK_WIDGET_SET_FLAGS (applybutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) select_directory_button, "clicked", G_CALLBACK (importsfmdialog_on_select_directory_button_clicked), gpointer(this));
  g_signal_connect ((gpointer) change_selection_button, "clicked", G_CALLBACK (on_change_selection_button_clicked), gpointer(this));
  g_signal_connect ((gpointer) backbutton, "clicked", G_CALLBACK (importsfmdialog_on_backbutton_clicked), gpointer(this));
  g_signal_connect ((gpointer) forwardbutton, "clicked", G_CALLBACK (importsfmdialog_on_forwardbutton_clicked), gpointer(this));
  g_signal_connect ((gpointer) applybutton, "clicked", G_CALLBACK (importsfmdialog_on_applybutton_clicked), gpointer(this));

  gtk_widget_grab_focus (forwardbutton);
  gtk_widget_grab_default (forwardbutton);

  set_gui();
}


ImportSfmDialog::~ImportSfmDialog ()
{
  // Clean up temporary files.
  remove_directory (temporary_file (TEMP_DIR));
  gtk_widget_destroy (importsfmdialog);
}


int ImportSfmDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (importsfmdialog));
}


ustring ImportSfmDialog::pick_newest_filename (const vector <ustring>& filenames)
/*
This function returns the newest filename of the vector of filenames given.
*/
{
  // Pick newest.
  unsigned int modificationtime = 0;
  ustring returnvalue;
  // Solve bug that when the modification times were equal, nothing was taken.
  if (filenames.size() > 0)
    returnvalue = filenames[0];
  // Get newest (= youngest) file.
  for (unsigned int i = 0; i < filenames.size (); i++) {
    unsigned int modtime = file_get_modification_time (filenames[i]);
    if (modtime > modificationtime) {
      modificationtime = modtime;
      returnvalue = filenames[i];
    }
  }
  return returnvalue;
}


void ImportSfmDialog::on_backbutton_clicked () 
{
  gtk_notebook_prev_page (GTK_NOTEBOOK (notebook));
  set_gui();
}


void ImportSfmDialog::on_forwardbutton_clicked () 
{
  gtk_notebook_next_page (GTK_NOTEBOOK (notebook));
  set_gui();
}


void ImportSfmDialog::on_select_directory_button_clicked ()
{
  GtkWidget *dialog;
  dialog = gtk_file_chooser_dialog_new ("Select a folder", GTK_WINDOW (importsfmdialog),
    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
    directory = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    select_directory_page();
  }
  gtk_widget_destroy (dialog);  
}


void ImportSfmDialog::set_gui()
{
  gtk_widget_set_sensitive (backbutton, TRUE);
  gtk_widget_set_sensitive (forwardbutton, TRUE);
  gtk_widget_set_sensitive (applybutton, FALSE);
  gint currentpage = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
  switch (currentpage)
  {
  case 0 :
    {
      gtk_widget_set_sensitive (backbutton, FALSE);
      break;
    }
  case 1 :
    {
      select_directory_page();
      break;
    }
  case 2 :
    {
      list_all_files_page();
      break;
    }
  case 3 :
    {
      summary_per_book_page();
      break;
    }
  case 4 :
    {
      // Screen is already in the notebook.
      break;
    }
  case 5 :
    {
      unicode_page();
      break;
    }
  case 6 :
    {
      // Screen is already in the notebook.
      break;
    }
  case 7 :
    {
      gtk_widget_set_sensitive (forwardbutton, FALSE);
      gtk_widget_set_sensitive (applybutton, TRUE);
      break;
    }
  };
}


void ImportSfmDialog::select_directory_page()
{
  WaitDialog wd (500, 450, 300);
  wd.run ();
  // Initialize variabele directory.
  if (directory.empty ()) directory = g_get_home_dir ();
  gtk_label_set_text (GTK_LABEL (label8), directory.c_str());
  // Read directory, and leave subdirectories out.
  vector <ustring> filenames;
  ReadFiles rd (directory, "", "");
  for (unsigned int i = 0; i < rd.files.size (); i++) {
    ustring filename = gw_build_filename (directory, rd.files[i]);
    /* Only check on files with a size smaller than so many bytes.
     * This solves a problem of importing files, when a huge file
     * of hundreds of megabytes is found. It tries to read it all.
     * This takes a very long time. Therefore only files that are
     * relative small are taken into account.
     */
    struct stat statbuf;
    stat (filename.c_str(), &statbuf);
    if (statbuf.st_size < 1000000)
      filenames.push_back (filename);
  }
  // Clear any previous data
  sfm_files.clear();
  // See which files seem to be the SFM ones.
  for (unsigned int i = 0; i < filenames.size (); i++)
    if (!get_id_from_file (filenames[i]).empty ())
      sfm_files.push_back (filenames[i]);
  // Info for user.
  if (filenames.size () == 0)
    gtk_label_set_text (GTK_LABEL (number_of_files_label), "This directory contains no files");
  else
    {
      ustring info = "Number of files in this directory: " + convert_to_string (int (filenames.size ()));
      info.append ("\nNumber of files that appear to be SFM texts: " + convert_to_string (int (sfm_files.size ())));
      gtk_label_set_text (GTK_LABEL (number_of_files_label), info.c_str());
    }
  // We only can go forward if we've found sfm texts.
  gtk_widget_set_sensitive (forwardbutton, sfm_files.size () > 0);
}


void ImportSfmDialog::list_all_files_page()
{
  ustring message;
  message.append("\n");
  for (unsigned int i = 0; i < sfm_files.size (); i++)
    message.append (sfm_files[i] + "\n");
  gtk_label_set_text (GTK_LABEL (list_of_files_label), message.c_str());
}


void ImportSfmDialog::summary_per_book_page()
{
  // Some visual candy...
  WaitDialog wd (500, 450, 300);
  wd.run ();
  // Clear any previously copied files.
  {
    ustring command = "rm -rf ";
    command.append (temporary_file (TEMP_DIR));
    command.append ("/*");
    system (command.c_str());
  }
  // Clear selection mechanism.
  selectable_books.clear();
  selected_books.clear();
  // This variabele shows whether we've some books.
  bool there_are_books_to_import = false;
  // Make a vector that contains all names of the smsfiles.
  vector <ustring> smsfiles_names;
  for (unsigned int i = 0; i < sfm_files.size (); i++)
    smsfiles_names.push_back (id_to_biblebook_english (get_id_from_file (sfm_files[i])));
  ustring message;
  Scripture scripture (myconfiguration->project);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
    ustring bookname = index_to_biblebook (i);
    message.append (bookname + "\n");
    bool inproject = false;
    for (unsigned int i2 = 0; i2 < scripture.books.size (); i2++) {
      if (scripture.books[i2] == bookname) {
        inproject = true;
        break;
      }
    }
    if (inproject)
      message.append (" - already in project ... skipping\n");
    else {
      // Search if the book is among the files.
      vector <ustring> filenames_particular_book;
      for (unsigned int i2 = 0; i2 < sfm_files.size (); i2++) {
        if (smsfiles_names[i2] == bookname) {
          message.append ("- we found " + sfm_files[i2] + "\n");
          filenames_particular_book.push_back (sfm_files[i2]);
        }
      }
      // Sort out what we found.
      ustring booktocopy;
      if (filenames_particular_book.empty ())
        message.append ("- could not be found among the files\n");
      else if (filenames_particular_book.size () == 1) {
        booktocopy = filenames_particular_book[0];
        message.append ("- we pick " + booktocopy + "\n");
      }
      else {
        booktocopy = pick_newest_filename (filenames_particular_book);
        message.append ("- we take the newest: " + booktocopy + "\n");
      }
      // If we've a book, copy it now.
      if (!booktocopy.empty ()) {
        ustring command = "cp '";
        command.append (booktocopy);
        command.append ("' '");
        command.append (temporary_file (TEMP_DIR));
        command.append ("/" + bookname);
        command.append ("'");
        system (command.c_str());
        there_are_books_to_import = true;
        // Add to selectable and selected books.
        selectable_books.push_back(bookname);
        selected_books.insert (bookname);
      }
    }
  }
  if (!there_are_books_to_import)
    message.append ("\nThere are no books that can be imported");
  gtk_label_set_text (GTK_LABEL (summary1label), message.c_str());
}


void ImportSfmDialog::unicode_page()
{
  // Some visual candy, because the process might take time.
  WaitDialog wd (500, 450, 300);
  wd.run ();
  // Check whether everything is in Unicode.
  ustring directory = temporary_file (TEMP_DIR);
  ReadFiles rd (directory, "", "");
  ustring encoding; // Encoding to be used for all the books.
  for (unsigned int i = 0; i < rd.files.size (); i++) {
    ustring filename = gw_build_filename (directory, rd.files[i]);
    Book book (filename, true, encoding);
    encoding = book.from_encoding;
  }
}


void ImportSfmDialog::on_change_selection ()
{
  SelectBooksDialog dialog (sbSearch, selectable_books, selected_books, myconfiguration, mysession);
  dialog.run ();
}


void ImportSfmDialog::on_apply()
{
  // Copy the files to the project.
  ustring directory = temporary_file (TEMP_DIR);
  ReadFiles rd (directory, "", "");
  for (unsigned int i = 0; i < rd.files.size (); i++) {
    // Copy only when book was selected.
    if (selected_books.find (rd.files[i]) != selected_books.end()) {
      ustring source = gw_build_filename (directory, rd.files[i]);
      ustring destination = projectutils_scripture_filename (myconfiguration->project, rd.files[i]);
      create_directory (g_path_get_dirname (destination.c_str()));
      ustring cmd = "cp '";
      cmd.append (source);
      cmd.append ("' '");
      cmd.append (destination);
      cmd.append ("'");
      system (cmd.c_str());
    }
  }
}


void ImportSfmDialog::importsfmdialog_on_backbutton_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ImportSfmDialog *) user_data)->on_backbutton_clicked();
}


void ImportSfmDialog::importsfmdialog_on_forwardbutton_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ImportSfmDialog *) user_data)->on_forwardbutton_clicked();
}


void ImportSfmDialog::importsfmdialog_on_applybutton_clicked (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ImportSfmDialog *) user_data)->on_apply();
}


void ImportSfmDialog::importsfmdialog_on_select_directory_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ImportSfmDialog *) user_data)->on_select_directory_button_clicked();
}


void ImportSfmDialog::on_change_selection_button_clicked (GtkButton *button, gpointer user_data)
{
  ((ImportSfmDialog *) user_data)->on_change_selection();
}
