/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogprojects.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "directories.h"
#include <glib.h>
#include "gtkwrappers.h"
#include "combobox.h"


ProjectsDialog::ProjectsDialog (const ustring& currentproject)
{
  projectsdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (projectsdialog), "Delete project");
  gtk_window_set_position (GTK_WINDOW (projectsdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (projectsdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (projectsdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  // Get all projects, and take the one currently opened out.
  vector < ustring > projectslist;
  Projects projects (directories_get_projects ());
  for (unsigned int i = 0; i < projects.projects.size (); i++)
    if (projects.projects[i] != currentproject)
      projectslist.push_back (projects.projects[i]);

  // Show info.
  if (projectslist.size () == 0)
    label1 = gtk_label_new ("There are no projects to delete");
  else
    label1 = gtk_label_new ("Select the project to delete and then press OK");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 8);

  label2 = gtk_label_new ("");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 8);

  if (!currentproject.empty ())
    gtk_label_set_text (GTK_LABEL (label2), "(the project that is opened is not shown)");

  combobox1 = gtk_combo_box_new_text ();
  gtk_widget_show (combobox1);
  gtk_box_pack_start (GTK_BOX (vbox1), combobox1, FALSE, FALSE, 0);
  // Fill the combo box with the projects that can be deleted.
  combobox_set_strings (combobox1, projectslist);
  if (projectslist.size () > 0)
    combobox_set_string (combobox1, projectslist[0]);

  dialog_action_area1 = GTK_DIALOG (projectsdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (projectsdialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (projectsdialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);
  if (projectslist.size () == 0) {
    // No projects, disable OK button.
    gtk_widget_set_sensitive (okbutton1, FALSE);
  }

  g_signal_connect ((gpointer) okbutton1, "clicked",
                    G_CALLBACK (projectsdialog_on_okbutton1_clicked),
                    gpointer(this));

  gtk_widget_grab_focus (combobox1);
  gtk_widget_grab_default (okbutton1);

}


ProjectsDialog::~ProjectsDialog ()
{
  gtk_widget_destroy (projectsdialog);
}


int ProjectsDialog::run()
{
  return gtk_dialog_run (GTK_DIALOG (projectsdialog));
}


void ProjectsDialog::projectsdialog_on_okbutton1_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
  ((ProjectsDialog *) user_data)->on_okbutton1_clicked();
}


void ProjectsDialog::on_okbutton1_clicked()
{
  int result;
  result = gtkw_dialog_question (projectsdialog, "Are you sure you want to delete this project?");
  if (result == GTK_RESPONSE_YES) {
    result = gtkw_dialog_question (projectsdialog, "Are you really sure to delete something worth perhaps years of work?");
  }
  if (result == GTK_RESPONSE_YES) {
    ustring project;
    project = combobox_get_active_string (combobox1);
    projectutils_delete_project (project);
  }
}
