/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "htmlbrowser.h"
#include <gtk/gtk.h>
#include "gtkwrappers.h"
#include "gwrappers.h"
#include "directories.h"
#include "utilities.h"
#include "unixwrappers.h"


#define NUMBER_OF_BROWSERS 8


ustring helpcommand (int index, bool& add_file___)
{
  // Returns a command to be executed to get the helpfile displayed.
  // Variable "index" selects which browser will be used.
  // Variable *add_file___" returns whether file:// should be added to the name.
  ustring command;
  switch (index)
    {
    case 0:
      {
        command = "htmlview";
        add_file___ = true;
        break;
      }
    case 1:
      {
        command = "firefox";
        add_file___ = true;
        break;
      }
    case 2:
      {
        command = "mozilla";
        add_file___ = true;
        break;
      }
    case 3:
      {
        command = "galeon";
        add_file___ = true;
        break;
      }
    case 4:
      {
        command = "konqueror";
        add_file___ = true;
        break;
      }
    case 5:
      {
        command = "epiphany";
        add_file___ = true;
        break;
      }
    case 6:
      {
        command = "opera";
        add_file___ = true;
        break;
      }
    case 7:
      {
        command = "open";
        add_file___ = false;
        break;
      }
      // Note: when adding new browsers, ensure that the variabele
      // NUMBER_OF_BROWSERS is updated too.
    }
  return command;
}


void htmlbrowser (const ustring& filename)
// Get the name of an existing browser, and view the file through it.
{
  int browser_index = -1;
  ustring outputfile;
  for (unsigned int i = 0; i < NUMBER_OF_BROWSERS; i++) {
    // At first we used the exit code of "which" to determine whether it had
    // found the program, but this does not work on BSD likes. So now we read
    // the output to see what whether it found the program.
    bool dummy;
    if (unix_which (helpcommand (i, dummy))) {
      browser_index = i;
      break;
    }
  }
  if (browser_index >= 0) {
    bool add_file;
    ustring command;
    command = helpcommand (browser_index, add_file);
    command.append (" '");
    if (add_file)
      command.append ("file://");
    command.append (filename);
    command.append ("' > /dev/null &");
    system (command.c_str ());
  } else {
    // No suitable browser was found.
    // Inform the user about it, and what to do to solve it.
    ustring message;
    message = "The help cannot be displayed, because there is no suitable browser installed.\n";
    message.append ("Install one of the following supported browsers:\n");
    for (unsigned int i = 0; i < NUMBER_OF_BROWSERS; i++) {
      bool dummy;
      message.append (helpcommand (i, dummy));
      message.append ("\n");
    }
    gtkw_dialog_error (NULL, message);
  }    
}
