/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef INCLUDED_NOTE_TEMPLATE_H
#define INCLUDED_NOTE_TEMPLATE_H


#include "libraries.h"
#include "types.h"


class NoteTemplate
// Manages settings for one note template.
{
public:
  NoteTemplate (const ustring& filename, NoteType notetype, bool write);
  virtual ~NoteTemplate ();
  int numbering;
  ustring anchor;
  bool automatic_reference;
  bool reference_chapter;
  ustring reference_division;
  bool reference_verse;
  ustring reference_suffix;
  vector<ustring> content_markers;
  vector<ustring> content_texts;
  vector<bool> content_remembers;
private:
  void load ();
  ustring myfilename;
  bool mywrite;
  ustring current_element;
  ustring current_value;
  ustring content_marker;
  ustring content_text;
  ustring content_remember;
  static void start_element_handler  (GMarkupParseContext *context,
                                      const gchar         *element_name,
                                      const gchar        **attribute_names,
                                      const gchar        **attribute_values,
                                      gpointer             user_data,
                                      GError             **error);
  virtual void start_element_handler (const ustring& element_name);
  static void end_element_handler    (GMarkupParseContext *context,
                                      const gchar         *element_name,
                                      gpointer             user_data,
                                      GError             **error);
  virtual void end_element_handler   (const ustring& element_name);
  static void text_handler           (GMarkupParseContext *context,
                                      const gchar         *text,
                                      gsize                text_len,
                                      gpointer             user_data,
                                      GError             **error);
  virtual void text_handler          (const ustring& text);
  static void passthrough_handler    (GMarkupParseContext *context,
                                      const gchar         *passthrough_text,
                                      gsize                text_len,
                                      gpointer             user_data,
                                      GError             **error);
  static void error_handler          (GMarkupParseContext *context,
                                      GError              *error,
                                      gpointer             user_data);
};


#endif
