/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <libgen.h>
#include <glib.h>
#include <config.h>
#include "gwrappers.h"
#include "gtkwrappers.h"
#include "scripturechecks.h"
#include "scripture.h"
#include "directories.h"
#include "references.h"
#include "project.h"
#include "scripturechecksloadresults.h"
#include "scripturechecksdisplayresultsbrowser.h"
#include "dialogcheck.h"
#include "style.h"
#include "stylesheetutils.h"
#include "shell.h"


ustring scripture_checks_internal_generate_filenames_combined (Configuration * configuration, Session * session)
{
  ustring filenames;
  Scripture scripture (configuration->project);
  for (unsigned int i = 0; i < scripture.paths.size(); i++) {
    if (session->selected_books.find (scripture.books[i]) != session->selected_books.end())
      filenames.append (gw_shell_quote (scripture.paths[i]) + " ");
  }    
  return filenames;
}


vector<ustring> scripture_checks_internal_generate_filenames_separate (Configuration * configuration, Session * session)
{
  vector<ustring> filenames;
  Scripture scripture (configuration->project);
  for (unsigned int i = 0; i < scripture.paths.size(); i++) {
    if (session->selected_books.find (scripture.books[i]) != session->selected_books.end())
      filenames.push_back (gw_shell_quote (scripture.paths[i]));
  }    
  return filenames;
}


ustring scripture_checks_internal_temporal_outputfile ()
// Produces the filename for the messages in the temporal directory.
{
  return gw_build_filename (directories_get_temp(), "check.out");
}


ustring scripture_checks_internal_temporal_errorfile ()
// Produces the filename for the errors in the temporal directory.
{
  return gw_build_filename (directories_get_temp(), "check.err");
}


void scripture_checks_internal_unlink_files ()
{
  unlink (scripture_checks_internal_temporal_outputfile ().c_str());
  unlink (scripture_checks_internal_temporal_errorfile ().c_str());
}


void scripture_checks_internal_display_results (GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Configuration * configuration)
// Displays the results of the checks to the user.
{
  // See if we have any error messages.
  gchar *contents;
  g_file_get_contents (scripture_checks_internal_temporal_errorfile().c_str(), &contents, NULL, NULL);
  ustring ucontents = contents;
  g_free (contents);
  if (!ucontents.empty()) {
    ucontents.insert (0, "There were some errors while checking:\n\n");
    gtkw_dialog_error (NULL, ucontents);
  }
  ScriptureChecksLoadResults sclr (scripture_checks_internal_temporal_outputfile ());
  References references (liststore, treeview, treecolumn);
  references.set_references (sclr.references, sclr.comments, configuration);
  references.fill_store ();
}


void scripture_checks_internal_display_results (CheckDialogType displaywhat)
// Displays the results of the checks to the user.
{
  // See if we have any error messages.
  gchar *contents;
  g_file_get_contents (scripture_checks_internal_temporal_errorfile().c_str(), &contents, NULL, NULL);
  ustring ucontents = contents;
  g_free (contents);
  if (!ucontents.empty()) {
    ucontents.insert (0, "There were some errors while checking:\n\n");
    gtkw_dialog_error (NULL, ucontents);
  }
  ScriptureChecksDisplayResultsBrowser scdrb (displaywhat, scripture_checks_internal_temporal_outputfile ());
}


void scripture_checks_chapters_verses (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtChaptersVerses, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  progress->start();
  scripture_checks_internal_unlink_files ();
  vector<ustring> filenames;
  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
  progress->set_iterate (0, 1, filenames.size());
  Project project (configuration->project);
  for (unsigned int i = 0; i < filenames.size(); i++) {
    progress->iterate ();
    if (progress->cancel)
      break;
    ustring command;
    command = "sc-input-usfm " + filenames[i];
    command.append (" | sc-chapters-verses --versification " + project.versification);
    command.append (" >> " + scripture_checks_internal_temporal_outputfile());
    command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
    system (command.c_str());
  }
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


void scripture_checks_count_usfms (Configuration * configuration, Session * session, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtMarkersCount, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  progress->start();
  scripture_checks_internal_unlink_files ();
  ustring filenames;
  filenames = scripture_checks_internal_generate_filenames_combined (configuration, session);
  progress->set_fraction (0.5);
  ustring command;
  command = "sc-count-usfm ";
  if (session->checkmarkercounttype == cmctSortUsfm)
    command.append ("--sort-on-usfm");
  if (session->checkmarkercounttype == cmctSortCount)
    command.append ("--sort-on-count");
  command.append (" " + filenames);
  command.append (" >> " + scripture_checks_internal_temporal_outputfile());
  command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
  system (command.c_str());
  progress->set_fraction (1);
  scripture_checks_internal_display_results (cdtMarkersCount);
  progress->finish();
}


void scripture_checks_validate_usfms (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  // Handle dialog for the user.
  int dialogresponse;
  {
    CheckDialog dialog (cdtMarkersValidate, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  // Progress information.
  progress->start();
  // Clear up old stuff.
  scripture_checks_internal_unlink_files ();
  // Get all filenames to check for.
  vector<ustring> filenames;
  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
  // Get all markers we have in the stylesheet.
  ustring markers;
  {
    Project project (configuration);
    vector<Style> styles;
    stylesheet_get_styles (project.get_stylesheet(), styles);
    for (unsigned int i = 0; i < styles.size(); i++) {
      if (!markers.empty())
        markers.append (" ");
      markers.append (styles[i].marker);
      if (styles[i].type == stInlineText) {
        markers.append (" " + styles[i].marker + "*");
      }
    }
  }
  // Progress.
  progress->set_iterate (0, 1, filenames.size());
  // Go through each filename and check it.
  for (unsigned int i = 0; i < filenames.size(); i++) {
    // Progress / cancel information.
    progress->iterate ();
    if (progress->cancel)
      break;
    // Run the program for the library, that does the actual checking.
    ustring command;
    command = "sc-input-usfm " + filenames[i]
      + " | sc-check-usfm --markers-in-stylesheet '" + markers + "'"
      + " >> " + scripture_checks_internal_temporal_outputfile()
      + " 2>> " + scripture_checks_internal_temporal_errorfile();
    system (command.c_str());
  }
  // Display results, and finish progress.
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


void scripture_checks_compare_usfms (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  // Open dialog to present information to user and to get the options from him.
  int dialogresponse;
  {
    CheckDialog dialog (cdtMarkersCompare, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  // Open the two scripture.
  Scripture scripture (configuration->project);
  Scripture scripture2 (configuration->check_markers_compare_project);
  // Container with flags to verify all books in scripture2 have been dealt with.
  vector<bool> scripture2_checked;
  for (unsigned int i = 0; i < scripture2.books.size(); i++)
    scripture2_checked.push_back (false);
  // Display progress.
  progress->start();
  scripture_checks_internal_unlink_files ();
  progress->set_iterate (0, 1, scripture.paths.size());
  // Go through each book we have in this scripture.
  for (unsigned int i = 0; i < scripture.paths.size(); i++) {
    progress->iterate ();
    if (progress->cancel)
      break;
    // See whether this book is included in the ones to check.
    if (session->selected_books.find (scripture.books[i]) != session->selected_books.end()) {
      // Get a pointer to the same book in the second scripture.
      int pointer2 = -1;
      for (unsigned int i2 = 0; i2 < scripture2.books.size(); i2++) {
        if (scripture.books[i] == scripture2.books[i2]) {
          pointer2 = i2;
          break;
        }
      }
      ustring command;
      if (pointer2 >= 0) {
        // Ok, same book found in second scripture.
        ustring file1 = gw_shell_quote (scripture.paths[i]);
        ustring file2 = gw_shell_quote (scripture2.paths[pointer2]);
        command = "sc-compare-usfm " + file1 + " " + file2;
        if ((!configuration->check_markers_compare_all_markers) && (!configuration->check_markers_compare_include_only.empty())) {
          command.append (" --include-only '" + configuration->check_markers_compare_include_only + "'");
        }
        if (!configuration->check_markers_compare_ignore.empty()) {
          command.append (" --ignore '" + configuration->check_markers_compare_ignore + "'");
        }
        if (configuration->check_markers_compare_ignore_verse_zero)
          command.append (" --ignore-verse-zero");
        // Mark this book as done.
        scripture2_checked [pointer2] = true;
      }
      else {
        // Same book not found: give message about that.
        command = "sc-message '" + scripture.books[i]
          + "' 0 0 'Extra book'";
      }
      command.append (" >> " + scripture_checks_internal_temporal_outputfile());
      command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
      system (command.c_str());
    }
  }
  // See whether there are still some unchecked books in the second scripture.
  // Give message if there are unchecked ones, but only in case ALL books
  // were selected for checking.
  if (scripture.books.size() == session->selected_books.size()) {
    for (unsigned int i = 0; i < scripture2_checked.size(); i++) {
      if (!scripture2_checked[i]) {
        ustring command;
        command = "sc-message '" + scripture2.books[i]
          + "' 0 0 'Missing book'";
        command.append (" >> " + scripture_checks_internal_temporal_outputfile());
        command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
        system (command.c_str());
      }
    }
  }
  // Display results and finish.
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


void scripture_checks_count_characters (Configuration * configuration, Session * session, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtCharactersCount, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  scripture_checks_internal_unlink_files ();
  ustring filenames;
  filenames = scripture_checks_internal_generate_filenames_combined (configuration, session);
  progress->set_fraction (0.5);
  ustring command;
  command = "sc-input-usfm --categorize ";
  command.append (filenames);
  command.append (" | sc-count-characters ");
  if (session->checkcharacterscounttype == chctNoSort)
    command.append ("--no-sort");
  if (session->checkcharacterscounttype == chctSortUsfm)
    command.append ("--sort-on-character");
  if (session->checkcharacterscounttype == chctSortCount)
    command.append ("--sort-on-count");
  command.append (" --invisible-characters");
  command.append (" >> " + scripture_checks_internal_temporal_outputfile());
  command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
  run_shell_progress (command, "sc-count-characters", progress, "Counting", false, 3000000);
  scripture_checks_internal_display_results (cdtCharactersCount);
}


void scripture_checks_unwanted_patterns (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtUnwantedPatterns, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  progress->start();
  scripture_checks_internal_unlink_files ();
  vector<ustring> filenames;
  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
  progress->set_iterate (0, 1, filenames.size());
  for (unsigned int i = 0; i < filenames.size(); i++) {
    progress->iterate ();
    if (progress->cancel)
      break;
    ustring command;
    command = "sc-input-usfm --categorize " + filenames[i];
    command.append (" | sc-unwanted-patterns '" + checks_unwanted_patterns_get_filename (configuration->project) + "'");    
    command.append (" >> " + scripture_checks_internal_temporal_outputfile());
    command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
    system (command.c_str());
cout << command << endl;
  }
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


void scripture_checks_capitalization (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtCapitalization, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  progress->start();
  scripture_checks_internal_unlink_files ();
  vector<ustring> filenames;
  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
  progress->set_iterate (0, 1, filenames.size());
  for (unsigned int i = 0; i < filenames.size(); i++) {
    progress->iterate ();
    if (progress->cancel)
      break;
    ustring command;
    command = "sc-input-usfm --categorize " + filenames[i];
    command.append (" | sc-capitalization");
    if (!configuration->check_capitalization_punctuation.empty())
      command.append (" --punctuation-followed-by-capitals '" + configuration->check_capitalization_punctuation + "'");
    if (!configuration->check_capitalization_ignore.empty())
      command.append (" --ignore-lowercase-following '" + configuration->check_capitalization_ignore + "'");
    command.append (" --abbreviations '" + checks_abbreviations_get_filename (configuration->project) + "'");    
    if (configuration->check_capitalization_allow_any_prefixes)
      command.append (" --any-prefixes");
    else
      command.append (" --uncapitalized-prefixes '" + checks_uncapitalized_prefixes_get_filename (configuration->project) + "'");    
    command.append (" --capitalized-suffixes '" + checks_capitalized_suffixes_get_filename (configuration->project) + "'");    
    command.append (" >> " + scripture_checks_internal_temporal_outputfile());
    command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
    system (command.c_str());
  }
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


void scripture_checks_repetition (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtRepetition, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  progress->start();
  scripture_checks_internal_unlink_files ();
  vector<ustring> filenames;
  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
  progress->set_iterate (0, 1, filenames.size());
  for (unsigned int i = 0; i < filenames.size(); i++) {
    progress->iterate ();
    if (progress->cancel)
      break;
    ustring command;
    command = "sc-input-usfm --categorize " + filenames[i];
    command.append (" | sc-repetition");
    if (configuration->check_repetition_ignore_case)
      command.append (" --ignore-case");
    if (configuration->check_repetition_show_only_these)
      command.append (" --show-only '" + checks_repetition_show_only_get_filename (configuration->project) + "'");
    if (configuration->check_repetition_ignore_these)
      command.append (" --ignore '" + checks_repetition_ignore_get_filename (configuration->project) + "'");
    command.append (" >> " + scripture_checks_internal_temporal_outputfile());
    command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
    system (command.c_str());
  }
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


void scripture_checks_matching_pairs (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtMatchingPairs, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  progress->start();
  scripture_checks_internal_unlink_files ();
  vector<ustring> filenames;
  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
  progress->set_iterate (0, 1, filenames.size());
  for (unsigned int i = 0; i < filenames.size(); i++) {
    progress->iterate ();
    if (progress->cancel)
      break;
    ustring command;
    command = "sc-input-usfm --categorize " + filenames[i];
    command.append (" | sc-matching-pairs");
    if (!configuration->check_matching_pairs_ignore.empty()) {
      command.append (" --ignore '");
      command.append (configuration->check_matching_pairs_ignore);
      command.append ("'");
    }
    command.append (" >> " + scripture_checks_internal_temporal_outputfile());
    command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
    system (command.c_str());
  }
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


void scripture_checks_unwanted_words (Configuration * configuration, Session * session, GtkListStore * liststore, GtkWidget * treeview, GtkTreeViewColumn * treecolumn, Progress * progress)
{
  int dialogresponse;
  {
    CheckDialog dialog (cdtUnwantedWords, configuration, session);
    dialogresponse = dialog.run();
  }
  if (dialogresponse != GTK_RESPONSE_OK)
    return;
  progress->start();
  scripture_checks_internal_unlink_files ();
  vector<ustring> filenames;
  filenames = scripture_checks_internal_generate_filenames_separate (configuration, session);
  progress->set_iterate (0, 1, filenames.size());
  for (unsigned int i = 0; i < filenames.size(); i++) {
    progress->iterate ();
    if (progress->cancel)
      break;
    ustring command;
    command = "sc-input-usfm --categorize " + filenames[i];
    command.append (" | sc-unwanted-words");
    command.append (" '" + checks_unwanted_words_get_filename (configuration->project) + "'");
    command.append (" >> " + scripture_checks_internal_temporal_outputfile());
    command.append (" 2>> " + scripture_checks_internal_temporal_errorfile());
    system (command.c_str());
  }
  scripture_checks_internal_display_results (liststore, treeview, treecolumn, configuration);
  progress->finish();
}


ustring checks_abbreviations_get_filename (const ustring& project)
{
  return gw_build_filename (directories_get_projects (), project, "abbreviations");
}


ustring checks_uncapitalized_prefixes_get_filename (const ustring& project)
{
  return gw_build_filename (directories_get_projects (), project, "uncapitalized-prefixes");
}


ustring checks_capitalized_suffixes_get_filename (const ustring& project)
{
  return gw_build_filename (directories_get_projects (), project, "capitalized_suffixes");
}


ustring checks_repetition_show_only_get_filename (const ustring& project)
{
  return gw_build_filename (directories_get_projects (), project, "repetitions_show_only");
}


ustring checks_repetition_ignore_get_filename (const ustring& project)
{
  return gw_build_filename (directories_get_projects (), project, "repetitions_ignore");
}


ustring checks_unwanted_patterns_get_filename (const ustring& project)
{
  return gw_build_filename (directories_get_projects (), project, "unwanted_patterns");
}


ustring checks_unwanted_words_get_filename (const ustring& project)
{
  return gw_build_filename (directories_get_projects (), project, "unwanted_words");
}
